@@ -82,8 +82,8 @@ mod tests {
         let input_size = image_dimensions.iter().product();
         let output_size = output_dimensions.iter().product();
 
-        let initializer = initializer::make_he();
-        let activation = activation::make_relu();
+        let initializer = initializer::he();
+        let activation = activation::relu();
         let mut l1 = Conv::new(
             (16, image_depth, 3, 3),
             (2, 2),
@@ -110,7 +110,7 @@ mod tests {
             let r1 = l1.forward(input);
             let r2 = l2.forward(r1);
 
-            let mut error = (&target - &r2).powf(2.0);
+            let error = (&target - &r2).powf(2.0);
 
             error.backward(None);
 
